require 'spec_helper'

describe NodeReportController do
  integrate_views

  def fixture(name)
    File.read(File.join(File.dirname(__FILE__), "../fixtures/reports/#{name}.yaml"))
  end

  describe "#show" do
    before :each do
      @baseline_report = Report.create_from_yaml(fixture('baseline'))
      @inspect_report  = Report.create_from_yaml(fixture('inspect'))

      @node = @inspect_report.node
      @node_baseline =
        NodeBaseline.create_from_report!(@baseline_report, @inspect_report.time)
      Summarizer.build_baseline_summary(@inspect_report.id, @node_baseline)
    end

    it "should work when the node doesn't have a baseline" do
      get :show, :id => Node.generate!

      response.should be_success

      assigns[:node_baseline].should be_nil
      assigns[:baseline_differences].should be_nil
      assigns[:unreviewed_changes_on_other_days].should == 0
      assigns[:other_days].should == []
    end

    it "should work selecting the date of the baseline" do
      get :show, :id => @node.id, :date => @node_baseline.date.to_s

      response.should be_success

      assigns[:date].should == @node_baseline.date
      assigns[:node_baseline].should_not be_nil
      assigns[:baseline_differences].should_not be_nil
      assigns[:unreviewed_changes_on_other_days].should == 0
      assigns[:other_days].should == []
    end

    it "should not show any data if there is no baseline for the date" do
      date = @node_baseline.date.yesterday
      NodeBaseline.find_by_node_id_and_date(@node.id, date).should be_nil

      get :show, :id => @node.id, :date => date.to_s

      response.should be_success

      assigns[:date].should == date
      assigns[:node_baseline].should == nil
      assigns[:baseline_difference].should == nil
      assigns[:unreviewed_changes_on_other_days].should == BaselineDifference.count
      assigns[:other_days].should be_present
    end

    it "should default to today if no date is specified" do
      get :show, :id => @node.id

      response.should be_success
      assigns[:date].should == Time.zone.now.to_date
      assigns[:node_baseline].should == nil
    end
  end

  describe "#failures" do
    before :each do
      @baseline_report = Report.create_from_yaml(fixture('baseline'))
      @inspect_report  = Report.create_from_yaml(fixture('inspect'))

      @failed_resource = @inspect_report.resource_statuses.first
      @failed_resource.update_attributes!(:failed => true)
      @inspect_report.logs << ReportLog.new(:message => "Inspecting #{@failed_resource.name} failed.")

      @node = @inspect_report.node
      @node_baseline =
        NodeBaseline.create_from_report!(@baseline_report, @inspect_report.time)
      Summarizer.build_baseline_summary(@inspect_report.id, @node_baseline)
    end

    it "should work when the node doesn't have a baseline" do
      get :failures, :id => Node.generate!
      warn assigns[:failure_count].zero?.inspect

      response.should redirect_to :action => :show, :id => params[:id], :date => Time.zone.now.to_date.to_s(:db)
    end

    it "should work selecting the date of the baseline" do
      get :failures, :id => @node.id, :date => @node_baseline.date.to_s

      response.should be_success

      res = @node_baseline.baseline.resources.find_by_resource_type_and_title(
        @failed_resource.resource_type,
        @failed_resource.title
      )

      assigns[:date].should == @node_baseline.date
      assigns[:node_baseline].should_not be_nil
      assigns[:failures].should =~ [
        [
          @failed_resource.name,
          res.properties.map { |e| [e.name, e.value] },
          ["Inspecting #{@failed_resource.name} failed."],
        ]
      ]
    end

    it "should not show any data if there is no baseline for the date" do
      date = @node_baseline.date.yesterday
      NodeBaseline.find_by_node_id_and_date(@node.id, date).should be_nil

      get :failures, :id => @node.id, :date => date.to_s

      response.should redirect_to :action => :show, :id => @node.id, :date => date.to_s(:db)
    end

    it "should default to today if no date is specified" do
      get :failures, :id => @node.id

      response.should redirect_to :action => :show, :id => @node.id, :date => Time.zone.now.to_date.to_s(:db)
      assigns[:date].should == Time.zone.now.to_date
      assigns[:node_baseline].should be_nil
    end
  end

  describe "#export" do
    before :each do
      @baseline_report = Report.create_from_yaml(fixture('baseline'))
      @inspect_report  = Report.create_from_yaml(fixture('inspect'))

      @node = @inspect_report.node
      @node_baseline =
        NodeBaseline.create_from_report!(@baseline_report, @inspect_report.time)
      Summarizer.build_baseline_summary(@inspect_report.id, @node_baseline)
    end

    it "should not die when requesting CSV data" do
      get :export, :format => 'csv', :id => @node.id, :date => @node_baseline.date.to_s
      response.should be_success
    end

    it "should not die when requesting non-existent CSV data" do
      get :export, :format => 'csv', :id => Node.generate!
      response.should be_success
    end
  end

  describe "#mark_as_reviewed" do
    before :each do
      @baseline_report = Report.create_from_yaml(fixture('baseline'))
      @inspect_report  = Report.create_from_yaml(fixture('inspect'))

      @node = @inspect_report.node

      @node_baseline = NodeBaseline.create_from_report!(@baseline_report, @inspect_report.time)

      Summarizer.build_baseline_summary(@inspect_report.id, @node_baseline)

      @diff = BaselineDifference.find_by_resource_name('File[/etc/hosts]')
    end

    it "should mark the corresponding set of differences as 'accepted'" do
      get :mark_as_reviewed, :diff_id => @diff.id, :state => 'accepted'

      response.should be_success
      BaselineDifference.find_all_by_accepted(true).should == [@diff]
      BaselineDifference.find_all_by_accepted(false).should_not be_present
    end

    it "should mark the corresponding set of differences as 'rejected'" do
      get :mark_as_reviewed, :diff_id => @diff.id, :state => 'rejected'

      response.should be_success
      BaselineDifference.find_all_by_accepted(true).should_not be_present
      BaselineDifference.find_all_by_accepted(false).should == [@diff]
    end

    it "should fail if a difference is not specified" do
      expect {
        get :mark_as_reviewed, :state => 'accepted'
      }.to raise_error(ActiveRecord::RecordNotFound, /Couldn't find BaselineDifference/)

      BaselineDifference.find_all_by_accepted(true).should_not be_present
      BaselineDifference.find_all_by_accepted(false).should_not be_present
    end

    it "should fail if given an invalid difference" do
      expect {
        get :mark_as_reviewed, :diff_id => 5, :state => 'accepted'
      }.to raise_error(ActiveRecord::RecordNotFound, /Couldn't find BaselineDifference/)

      BaselineDifference.find_all_by_accepted(true).should_not be_present
      BaselineDifference.find_all_by_accepted(false).should_not be_present
    end

    it "should fail if setting state to anything but 'accepted' or 'rejected'" do
      get :mark_as_reviewed, :diff_id => @diff.id, :state => 'awesome'

      response.should_not be_success
      response.code.should == '400'
      response.body.should =~ /'awesome' is not an acceptable state for a BaselineDifference/

      BaselineDifference.find_all_by_accepted(true).should_not be_present
      BaselineDifference.find_all_by_accepted(false).should_not be_present
    end

    describe 'all changes in a contiguous block of differences' do
      before do
        @node_baselines = (1..5).map do |i|
          NodeBaseline.create_from_report!(@baseline_report, @inspect_report.time - i.days).tap do |b|
            b.differences = @node_baseline.differences.map(&:clone)
            b.save!
          end
        end.unshift(@node_baseline)

        @expected = @node_baselines.map do |bl|
          bl.differences.find_by_resource_name('File[/etc/hosts]')
        end
      end

      it "should be marked as 'accepted' when state is 'accepted'" do
        diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

        get :mark_as_reviewed, :diff_id => diff.id, :state => 'accepted'

        response.should be_success
        BaselineDifference.find_all_by_accepted(true).should =~ @expected
        BaselineDifference.find_all_by_accepted(false).should_not be_present
      end

      it "should be marked as 'rejected' when state is 'rejected" do
        diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

        get :mark_as_reviewed, :diff_id => diff.id, :state => 'rejected'

        response.should be_success
        BaselineDifference.find_all_by_accepted(true).should_not be_present
        BaselineDifference.find_all_by_accepted(false).should =~ @expected
      end

      describe 'where the discontinuity is not the beginning of time' do
        before do
          @node_baselines[-2].differences.find_by_resource_name('File[/etc/hosts]').update_attributes! :resource_status_digest => ''

          @expected = @node_baselines[0...-2].map do |bl|
            bl.differences.find_by_resource_name('File[/etc/hosts]')
          end
        end

        it "should mark only the changes in a contiguous block of differences as 'accepted'" do
            diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

            get :mark_as_reviewed, :diff_id => diff.id, :state => 'accepted'

            response.should be_success
            BaselineDifference.find_all_by_accepted(true).should =~ @expected
            BaselineDifference.find_all_by_accepted(false).should_not be_present
        end

        it "should mark only the changes in a contiguous block of differences as 'rejected'" do
            diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

            get :mark_as_reviewed, :diff_id => diff.id, :state => 'rejected'

            response.should be_success
            BaselineDifference.find_all_by_accepted(true).should_not be_present
            BaselineDifference.find_all_by_accepted(false).should =~ @expected
        end
      end

      describe 'where the action is not against the most recent change' do
        before do
          @node_baselines[2].differences.find_by_resource_name('File[/etc/hosts]').update_attributes! :resource_status_digest => ''

          @expected = @node_baselines[3..-1].map do |bl|
            bl.differences.find_by_resource_name('File[/etc/hosts]')
          end
        end

        it "should mark only the changes in a contiguous block of differences as 'accepted'" do
            diff = @node_baselines[3].differences.find_by_resource_name('File[/etc/hosts]')

            get :mark_as_reviewed, :diff_id => diff.id, :state => 'accepted'

            response.should be_success
            BaselineDifference.find_all_by_accepted(true).should =~ @expected
            BaselineDifference.find_all_by_accepted(false).should_not be_present
        end

        it "should mark only the changes in a contiguous block of differences as 'rejected'" do
            diff = @node_baselines[3].differences.find_by_resource_name('File[/etc/hosts]')

            get :mark_as_reviewed, :diff_id => diff.id, :state => 'rejected'

            response.should be_success
            BaselineDifference.find_all_by_accepted(true).should_not be_present
            BaselineDifference.find_all_by_accepted(false).should =~ @expected
        end
      end

      describe 'where the contiguous block contains previously accepted changes' do
        before do
          @old = 3.days.ago

          @node_baselines[-2].differences.find_by_resource_name('File[/etc/hosts]').update_attributes! :resource_status_digest => ''
          @node_baselines[-3].differences.find_by_resource_name('File[/etc/hosts]').update_attributes! :decided_at => @old, :accepted => true

          @diffs = @node_baselines.map do |bl|
            bl.differences.find_by_resource_name('File[/etc/hosts]')
          end
        end

        it "should mark only the changes in a contiguous block of differences as 'accepted'" do
          diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

          get :mark_as_reviewed, :diff_id => diff.id, :state => 'accepted'
          response.should be_success

          diff.reload
          @new = diff.decided_at.to_i
          @old = @old.to_i

          @diffs.map{|x| x.reload; [x.accepted, x.decided_at.to_i] }.
            should == [
                       [true, @new],
                       [true, @new],
                       [true, @new],
                       [true, @old],
                       [nil, 0],
                       [nil, 0]
                      ]
        end

        it "should mark only the changes in a contiguous block of differences as 'rejected'" do
          diff = @node_baseline.differences.find_by_resource_name('File[/etc/hosts]')

          get :mark_as_reviewed, :diff_id => diff.id, :state => 'rejected'
          response.should be_success

          diff.reload
          @new = diff.decided_at.to_i
          @old = @old.to_i

          @diffs.map{|x| x.reload; [x.accepted, x.decided_at.to_i] }.
            should == [
                       [false, @new],
                       [false, @new],
                       [false, @new],
                       [true, @old],
                       [nil, 0],
                       [nil, 0]
                      ]
        end
      end
    end
  end
end
